(*
This demo makes use of a couple of interesting hacks to fully
customize API messageboxes.  You can modify fonts, button
captions and even change the icon to your application's icon
without adding a byte of extra resource data to your project.

This was developed out of necessity since I do very little
work with Delphi's visual units.  Most of my current projects
require fairly strict adherence to the API.

It adds a nice touch and makes WINCIM's connection messagebox,
which was my inspiration, look positively primitive. (WINCIM
uses a resource to draw the messagebox...sorry folks but I
couldn't resist loading it into Resource Workshop to find out
for sure.)

Oh, and does it work in 32 bit?  Ahem...well...actually this
project was converted *backward* from a 32 bit project.

's Lea'sware from Cub

cublea@cyberstore.ca

Totally use-at-your-own-risk stuff.

*)

program Generic;

{$R GENERIC}

{Note that the Generic app is a Delphi demo app.
I use it as a testbed for dialog-related stuff}

uses WinTypes, WinProcs, Messages,Sysutils;

const
  AppName = 'Generic';

const
  idm_About = 100;


function About(Dialog: HWnd; Message, WParam: Word;
  LParam: Longint): Bool; export;
var MsgboxHwnd : HWND;
	Fonthandle:HFont;
	DlgFont:TLOGFONT;
	NewFont: HFont;
	NewIcon : THandle;
	ErrorID,ItemID: Integer;
begin
  About := True;
  case Message of
	wm_InitDialog:
	  Exit;
	wm_Command: begin
	  if (WParam = id_Cancel) then begin
		ItemID := ItemID + 1;
		EndDialog(Dialog, 1);
		Exit;
	  end;
	  If (WParam = id_ok) then begin
		  {Now, this part came from an unknown IRC user.
			I assume the PostMessage command is designed to pass
			the message to the messagebox-change procedure, and
			the fact that it's posted, not sent, allows the messagebox
			to be drawn before any of the WM_USER+1024 commands are
			processed.}
		  PostMessage(Dialog, WM_USER + 1024, 0, 0);
		  MessageBox(Dialog, 'text', 'caption', MB_OKCancel or MB_ICONQUESTION);
	  end;
	end;
{When the WM_USER+1024 message is sent to the window, that's when we want our
 modifications to be made.}
	wm_user + 1024: begin
{Now, when we fetch the messagebox windowhandle, we need to know its
 caption when using FindWindow. There's probably another way; I don't know it.
 The original author used FindWindow(MAKEINTRESOURCE(WC_DIALOG), 'caption')
 to insure that only dialog windows were polled by FindWindow, but when I
 tried FindWindow(nil,'My Message Caption') it worked just as well.}
		MsgboxHwnd := FindWindow(MAKEINTRESOURCE(WC_DIALOG), 'caption');
		if (MsgboxHwnd <> 0) then begin
{First let's mess with the button caption text.}
			SetDlgItemText(
				Msgboxhwnd,
				ID_OK {the ID number of the OK button},
				'Don''t press me' {the new button captio}
				);
{You could also modify the Cancel button text by specifying ID_CANCEL
	in another SetDlgItemText() command.}

{Now let's modify a button font.}
			Fonthandle := SendMessage(MsgboxHwnd,WM_GETFONT,0,0);
			GetObject(Fonthandle,sizeof(TLOGFONT), @Dlgfont);
			strpcopy(Dlgfont.lfFaceName,'Times');
{ You can also try messing with any other TLOGFONT characteristics.}
{			Dlgfont.lfClipPrecision := CLIP_LH_ANGLES or CLIP_TT_ALWAYS;
			Dlgfont.Lfweight := FW_BOLD;
			DlgFont.lfheight:= 25;
			Dlgfont.lfwidth:= 7;}
			NewFont := CreateFontIndirect(DlgFont);
			If NewFont <> 0 then SendDlgItemMessage(Msgboxhwnd,ID_OK,WM_SETFONT,NewFont,0);
{Finally let's apply the application's icon to the messagebox.
 Delphi uses MAINICON as the default. You can specify other icons in your app's
 resources as well. The hard part of this was finding the messagebox item ID for
 the icon. I hacked a little test program together and discovered by trial-and-
 error that its ID is 10.  (Sometimes you get lucky that way...it could have been
 1024, and I was incrementing by 1's when inputting sequential item IDs.)}
			NewIcon := LoadIcon(hinstance,'MAINICON');
			If Newicon <> 0 Then SendDlgItemMessage(Msgboxhwnd,10,STM_SETICON,NewIcon,0);
		end;
	end;
  end;
  About := False;
end;

function WindowProc(Window: HWnd; Message, WParam: Word;
  LParam: Longint): Longint; export;
var
  AboutProc: TFarProc;
begin
  WindowProc := 0;
  case Message of
	wm_Command:
	  if WParam = idm_About then
	  begin
		AboutProc := MakeProcInstance(@About, HInstance);
		DialogBox(HInstance, 'AboutBox', Window, AboutProc);
		FreeProcInstance(AboutProc);
		Exit;
	  end;
	wm_Destroy:
	  begin
		PostQuitMessage(0);
		Exit;
	  end;
  end;
  WindowProc := DefWindowProc(Window, Message, WParam, LParam);
end;

procedure WinMain;
var
  Window: HWnd;
  Message: TMsg;
const
  WindowClass: TWndClass = (
    style: 0;
    lpfnWndProc: @WindowProc;
    cbClsExtra: 0;
    cbWndExtra: 0;
    hInstance: 0;
	hIcon: 0;
    hCursor: 0;
    hbrBackground: 0;
    lpszMenuName: AppName;
    lpszClassName: AppName);
begin
  if HPrevInst = 0 then
  begin
	WindowClass.hInstance := HInstance;
    WindowClass.hIcon := LoadIcon(0, idi_Application);
	WindowClass.hCursor := LoadCursor(0, idc_Arrow);
    WindowClass.hbrBackground := GetStockObject(white_Brush);
    if not RegisterClass(WindowClass) then Halt(255);
  end;
  Window := CreateWindow(
    AppName,
    'Delphi Generic',
    ws_OverlappedWindow,
    cw_UseDefault,
    cw_UseDefault,
    cw_UseDefault,
    cw_UseDefault,
    0,
    0,
    HInstance,
    nil);
  ShowWindow(Window, CmdShow);
  UpdateWindow(Window);
  while GetMessage(Message, 0, 0, 0) do
  begin
    TranslateMessage(Message);
    DispatchMessage(Message);
  end;
  Halt(Message.wParam);
end;

begin
  WinMain;
end.
